@@ -18,6 +18,7 @@
 // *************************************************************************
 
 use std::fmt;
+use std::mem;
 use std::result;
 use std::thread;
 use std::time;
@@ -34,6 +35,7 @@ use crate::args;
 use crate::error;
 use crate::error::Error;
 use crate::pinentry;
+use crate::ExecCtx;
 use crate::Result;
 
 /// Create an `error::Error` with an error message of the format `msg: err`.
@@ -42,7 +44,7 @@ fn get_error(msg: &'static str, err: nitrokey::Error) -> Error {
 }
 
 /// Set `libnitrokey`'s log level based on the execution context's verbosity.
-fn set_log_level(ctx: &mut args::ExecCtx<'_>) {
+fn set_log_level(ctx: &mut ExecCtx<'_>) {
   let log_lvl = match ctx.verbosity {
     // The error log level is what libnitrokey uses by default. As such,
     // there is no harm in us setting that as well when the user did not
@@ -58,9 +60,9 @@ fn set_log_level(ctx: &mut args::ExecCtx<'_>) {
 }
 
 /// Connect to any Nitrokey device and do something with it.
-fn with_device<F>(ctx: &mut args::ExecCtx<'_>, op: F) -> Result<()>
+fn with_device<F>(ctx: &mut ExecCtx<'_>, op: F) -> Result<()>
 where
-  F: FnOnce(&mut args::ExecCtx<'_>, nitrokey::DeviceWrapper<'_>) -> Result<()>,
+  F: FnOnce(&mut ExecCtx<'_>, nitrokey::DeviceWrapper<'_>) -> Result<()>,
 {
   let mut manager = nitrokey::take()?;
   set_log_level(ctx);
@@ -79,9 +81,9 @@ where
 }
 
 /// Connect to a Nitrokey Storage device and do something with it.
-fn with_storage_device<F>(ctx: &mut args::ExecCtx<'_>, op: F) -> Result<()>
+fn with_storage_device<F>(ctx: &mut ExecCtx<'_>, op: F) -> Result<()>
 where
-  F: FnOnce(&mut args::ExecCtx<'_>, nitrokey::Storage<'_>) -> Result<()>,
+  F: FnOnce(&mut ExecCtx<'_>, nitrokey::Storage<'_>) -> Result<()>,
 {
   let mut manager = nitrokey::take()?;
   set_log_level(ctx);
@@ -102,12 +104,12 @@ where
 
 /// Connect to any Nitrokey device, retrieve a password safe handle, and
 /// do something with it.
-fn with_password_safe<F>(ctx: &mut args::ExecCtx<'_>, mut op: F) -> Result<()>
+fn with_password_safe<F>(ctx: &mut ExecCtx<'_>, mut op: F) -> Result<()>
 where
-  F: FnMut(&mut args::ExecCtx<'_>, nitrokey::PasswordSafe<'_, '_>) -> Result<()>,
+  F: FnMut(&mut ExecCtx<'_>, nitrokey::PasswordSafe<'_, '_>) -> Result<()>,
 {
   with_device(ctx, |ctx, mut device| {
-    let pin_entry = pinentry::PinEntry::from(pinentry::PinType::User, &device)?;
+    let pin_entry = pinentry::PinEntry::from(args::PinType::User, &device)?;
     try_with_pin_and_data(
       ctx,
       &pin_entry,
@@ -129,15 +131,15 @@ where
 ///
 /// If an error occurs, the error message `msg` is used.
 fn authenticate<'mgr, D, A, F>(
-  ctx: &mut args::ExecCtx<'_>,
+  ctx: &mut ExecCtx<'_>,
   device: D,
-  pin_type: pinentry::PinType,
+  pin_type: args::PinType,
   msg: &'static str,
   op: F,
 ) -> Result<A>
 where
   D: Device<'mgr>,
-  F: FnMut(&mut args::ExecCtx<'_>, D, &str) -> result::Result<A, (D, nitrokey::Error)>,
+  F: FnMut(&mut ExecCtx<'_>, D, &str) -> result::Result<A, (D, nitrokey::Error)>,
 {
   let pin_entry = pinentry::PinEntry::from(pin_type, &device)?;
 
@@ -145,34 +147,28 @@ where
 }
 
 /// Authenticate the given device with the user PIN.
-fn authenticate_user<'mgr, T>(
-  ctx: &mut args::ExecCtx<'_>,
-  device: T,
-) -> Result<nitrokey::User<'mgr, T>>
+fn authenticate_user<'mgr, T>(ctx: &mut ExecCtx<'_>, device: T) -> Result<nitrokey::User<'mgr, T>>
 where
   T: Device<'mgr>,
 {
   authenticate(
     ctx,
     device,
-    pinentry::PinType::User,
+    args::PinType::User,
     "Could not authenticate as user",
     |_ctx, device, pin| device.authenticate_user(pin),
   )
 }
 
 /// Authenticate the given device with the admin PIN.
-fn authenticate_admin<'mgr, T>(
-  ctx: &mut args::ExecCtx<'_>,
-  device: T,
-) -> Result<nitrokey::Admin<'mgr, T>>
+fn authenticate_admin<'mgr, T>(ctx: &mut ExecCtx<'_>, device: T) -> Result<nitrokey::Admin<'mgr, T>>
 where
   T: Device<'mgr>,
 {
   authenticate(
     ctx,
     device,
-    pinentry::PinType::Admin,
+    args::PinType::Admin,
     "Could not authenticate as admin",
     |_ctx, device, pin| device.authenticate_admin(pin),
   )
@@ -206,14 +202,14 @@ fn get_volume_status(status: &nitrokey::VolumeStatus) -> &'static str {
 /// second or third try, it will call `op` with the data returned by the
 /// previous call to `op`.
 fn try_with_pin_and_data_with_pinentry<D, F, R, E>(
-  ctx: &mut args::ExecCtx<'_>,
+  ctx: &mut ExecCtx<'_>,
   pin_entry: &pinentry::PinEntry,
   msg: &'static str,
   data: D,
   mut op: F,
 ) -> Result<R>
 where
-  F: FnMut(&mut args::ExecCtx<'_>, D, &str) -> result::Result<R, (D, E)>,
+  F: FnMut(&mut ExecCtx<'_>, D, &str) -> result::Result<R, (D, E)>,
   E: error::TryInto<nitrokey::Error>,
 {
   let mut data = data;
@@ -246,22 +242,22 @@ where
 
 /// Try to execute the given function with a PIN.
 fn try_with_pin_and_data<D, F, R, E>(
-  ctx: &mut args::ExecCtx<'_>,
+  ctx: &mut ExecCtx<'_>,
   pin_entry: &pinentry::PinEntry,
   msg: &'static str,
   data: D,
   mut op: F,
 ) -> Result<R>
 where
-  F: FnMut(&mut args::ExecCtx<'_>, D, &str) -> result::Result<R, (D, E)>,
+  F: FnMut(&mut ExecCtx<'_>, D, &str) -> result::Result<R, (D, E)>,
   E: Into<Error> + error::TryInto<nitrokey::Error>,
 {
   let pin = match pin_entry.pin_type() {
     // Ideally we would not clone here, but that would require us to
     // restrict op to work with an immutable ExecCtx, which is not
     // possible given that some clients print data.
-    pinentry::PinType::Admin => ctx.admin_pin.clone(),
-    pinentry::PinType::User => ctx.user_pin.clone(),
+    args::PinType::Admin => ctx.admin_pin.clone(),
+    args::PinType::User => ctx.user_pin.clone(),
   };
 
   if let Some(pin) = pin {
@@ -282,7 +278,7 @@ where
 /// This function behaves exactly as `try_with_pin_and_data`, but
 /// it refrains from passing any data to it.
 fn try_with_pin<F, E>(
-  ctx: &mut args::ExecCtx<'_>,
+  ctx: &mut ExecCtx<'_>,
   pin_entry: &pinentry::PinEntry,
   msg: &'static str,
   mut op: F,
@@ -297,10 +293,7 @@ where
 }
 
 /// Pretty print the status of a Nitrokey Storage.
-fn print_storage_status(
-  ctx: &mut args::ExecCtx<'_>,
-  status: &nitrokey::StorageStatus,
-) -> Result<()> {
+fn print_storage_status(ctx: &mut ExecCtx<'_>, status: &nitrokey::StorageStatus) -> Result<()> {
   println!(
     ctx,
     r#"  Storage:
@@ -331,7 +324,7 @@ fn print_storage_status(
 
 /// Query and pretty print the status that is common to all Nitrokey devices.
 fn print_status(
-  ctx: &mut args::ExecCtx<'_>,
+  ctx: &mut ExecCtx<'_>,
   model: &'static str,
   device: &nitrokey::DeviceWrapper<'_>,
 ) -> Result<()> {
@@ -366,7 +359,7 @@ fn print_status(
 }
 
 /// Inquire the status of the nitrokey.
-pub fn status(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
+pub fn status(ctx: &mut ExecCtx<'_>) -> Result<()> {
   with_device(ctx, |ctx, device| {
     let model = match device {
       nitrokey::DeviceWrapper::Pro(_) => "Pro",
@@ -377,7 +370,7 @@ pub fn status(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
 }
 
 /// List the attached Nitrokey devices.
-pub fn list(ctx: &mut args::ExecCtx<'_>, no_connect: bool) -> Result<()> {
+pub fn list(ctx: &mut ExecCtx<'_>, no_connect: bool) -> Result<()> {
   set_log_level(ctx);
 
   let device_infos = nitrokey::list_devices()?;
@@ -415,9 +408,9 @@ pub fn list(ctx: &mut args::ExecCtx<'_>, no_connect: bool) -> Result<()> {
 }
 
 /// Perform a factory reset.
-pub fn reset(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
+pub fn reset(ctx: &mut ExecCtx<'_>) -> Result<()> {
   with_device(ctx, |ctx, mut device| {
-    let pin_entry = pinentry::PinEntry::from(pinentry::PinType::Admin, &device)?;
+    let pin_entry = pinentry::PinEntry::from(args::PinType::Admin, &device)?;
 
     // To force the user to enter the admin PIN before performing a
     // factory reset, we clear the pinentry cache for the admin PIN.
@@ -439,12 +432,9 @@ pub fn reset(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
 }
 
 /// Change the configuration of the unencrypted volume.
-pub fn unencrypted_set(
-  ctx: &mut args::ExecCtx<'_>,
-  mode: args::UnencryptedVolumeMode,
-) -> Result<()> {
+pub fn unencrypted_set(ctx: &mut ExecCtx<'_>, mode: args::UnencryptedVolumeMode) -> Result<()> {
   with_storage_device(ctx, |ctx, mut device| {
-    let pin_entry = pinentry::PinEntry::from(pinentry::PinType::Admin, &device)?;
+    let pin_entry = pinentry::PinEntry::from(args::PinType::Admin, &device)?;
     let mode = match mode {
       args::UnencryptedVolumeMode::ReadWrite => nitrokey::VolumeMode::ReadWrite,
       args::UnencryptedVolumeMode::ReadOnly => nitrokey::VolumeMode::ReadOnly,
@@ -464,9 +454,9 @@ pub fn unencrypted_set(
 }
 
 /// Open the encrypted volume on the Nitrokey.
-pub fn encrypted_open(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
+pub fn encrypted_open(ctx: &mut ExecCtx<'_>) -> Result<()> {
   with_storage_device(ctx, |ctx, mut device| {
-    let pin_entry = pinentry::PinEntry::from(pinentry::PinType::User, &device)?;
+    let pin_entry = pinentry::PinEntry::from(args::PinType::User, &device)?;
 
     // We may forcefully close a hidden volume, if active, so be sure to
     // flush caches to disk.
@@ -479,7 +469,7 @@ pub fn encrypted_open(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
 }
 
 /// Close the previously opened encrypted volume.
-pub fn encrypted_close(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
+pub fn encrypted_close(ctx: &mut ExecCtx<'_>) -> Result<()> {
   with_storage_device(ctx, |_ctx, mut device| {
     // Flush all filesystem caches to disk. We are mostly interested in
     // making sure that the encrypted volume on the Nitrokey we are
@@ -494,7 +484,7 @@ pub fn encrypted_close(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
 }
 
 /// Create a hidden volume.
-pub fn hidden_create(ctx: &mut args::ExecCtx<'_>, slot: u8, start: u8, end: u8) -> Result<()> {
+pub fn hidden_create(ctx: &mut ExecCtx<'_>, slot: u8, start: u8, end: u8) -> Result<()> {
   with_storage_device(ctx, |ctx, mut device| {
     let pwd_entry = pinentry::PwdEntry::from(&device)?;
     let pwd = if let Some(pwd) = &ctx.password {
@@ -513,7 +503,7 @@ pub fn hidden_create(ctx: &mut args::ExecCtx<'_>, slot: u8, start: u8, end: u8)
 }
 
 /// Open a hidden volume.
-pub fn hidden_open(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
+pub fn hidden_open(ctx: &mut ExecCtx<'_>) -> Result<()> {
   with_storage_device(ctx, |ctx, mut device| {
     let pwd_entry = pinentry::PwdEntry::from(&device)?;
     let pwd = if let Some(pwd) = &ctx.password {
@@ -536,7 +526,7 @@ pub fn hidden_open(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
 }
 
 /// Close a previously opened hidden volume.
-pub fn hidden_close(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
+pub fn hidden_close(ctx: &mut ExecCtx<'_>) -> Result<()> {
   with_storage_device(ctx, |_ctx, mut device| {
     unsafe { sync() };
 
@@ -555,7 +545,7 @@ fn format_option<T: fmt::Display>(option: Option<T>) -> String {
 }
 
 /// Read the Nitrokey configuration.
-pub fn config_get(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
+pub fn config_get(ctx: &mut ExecCtx<'_>) -> Result<()> {
   with_device(ctx, |ctx, device| {
     let config = device
       .get_config()
@@ -577,13 +567,18 @@ pub fn config_get(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
 }
 
 /// Write the Nitrokey configuration.
-pub fn config_set(
-  ctx: &mut args::ExecCtx<'_>,
-  numlock: args::ConfigOption<u8>,
-  capslock: args::ConfigOption<u8>,
-  scrollock: args::ConfigOption<u8>,
-  user_password: Option<bool>,
-) -> Result<()> {
+pub fn config_set(ctx: &mut ExecCtx<'_>, args: args::ConfigSetArgs) -> Result<()> {
+  let numlock = args::ConfigOption::try_from(args.no_numlock, args.numlock, "numlock")?;
+  let capslock = args::ConfigOption::try_from(args.no_capslock, args.capslock, "capslock")?;
+  let scrollock = args::ConfigOption::try_from(args.no_scrollock, args.scrollock, "scrollock")?;
+  let otp_pin = if args.otp_pin {
+    Some(true)
+  } else if args.no_otp_pin {
+    Some(false)
+  } else {
+    None
+  };
+
   with_device(ctx, |ctx, device| {
     let mut device = authenticate_admin(ctx, device)?;
     let config = device
@@ -593,7 +588,7 @@ pub fn config_set(
       numlock: numlock.or(config.numlock),
       capslock: capslock.or(config.capslock),
       scrollock: scrollock.or(config.scrollock),
-      user_password: user_password.unwrap_or(config.user_password),
+      user_password: otp_pin.unwrap_or(config.user_password),
     };
     device
       .write_config(config)
@@ -602,7 +597,7 @@ pub fn config_set(
 }
 
 /// Lock the Nitrokey device.
-pub fn lock(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
+pub fn lock(ctx: &mut ExecCtx<'_>) -> Result<()> {
   with_device(ctx, |_ctx, mut device| {
     device
       .lock()
@@ -630,7 +625,7 @@ fn get_unix_timestamp() -> Result<u64> {
 
 /// Generate a one-time password on the Nitrokey device.
 pub fn otp_get(
-  ctx: &mut args::ExecCtx<'_>,
+  ctx: &mut ExecCtx<'_>,
   slot: u8,
   algorithm: args::OtpAlgorithm,
   time: Option<u64>,
@@ -693,16 +688,18 @@ fn prepare_base32_secret(secret: &str) -> Result<String> {
 }
 
 /// Configure a one-time password slot on the Nitrokey device.
-pub fn otp_set(
-  ctx: &mut args::ExecCtx<'_>,
-  mut data: nitrokey::OtpSlotData,
-  algorithm: args::OtpAlgorithm,
-  counter: u64,
-  time_window: u16,
-  secret_format: args::OtpSecretFormat,
-) -> Result<()> {
+pub fn otp_set(ctx: &mut ExecCtx<'_>, mut args: args::OtpSetArgs) -> Result<()> {
+  let mut data = nitrokey::OtpSlotData {
+    number: args.slot,
+    name: mem::take(&mut args.name),
+    secret: mem::take(&mut args.secret),
+    mode: args.digits.into(),
+    use_enter: false,
+    token_id: None,
+  };
+
   with_device(ctx, |ctx, device| {
-    let secret = match secret_format {
+    let secret = match args.format {
       args::OtpSecretFormat::Ascii => prepare_ascii_secret(&data.secret)?,
       args::OtpSecretFormat::Base32 => prepare_base32_secret(&data.secret)?,
       args::OtpSecretFormat::Hex => {
@@ -720,9 +717,9 @@ pub fn otp_set(
     };
     let data = nitrokey::OtpSlotData { secret, ..data };
     let mut device = authenticate_admin(ctx, device)?;
-    match algorithm {
-      args::OtpAlgorithm::Hotp => device.write_hotp_slot(data, counter),
-      args::OtpAlgorithm::Totp => device.write_totp_slot(data, time_window),
+    match args.algorithm {
+      args::OtpAlgorithm::Hotp => device.write_hotp_slot(data, args.counter),
+      args::OtpAlgorithm::Totp => device.write_totp_slot(data, args.time_window),
     }
     .map_err(|err| get_error("Could not write OTP slot", err))?;
     Ok(())
@@ -730,11 +727,7 @@ pub fn otp_set(
 }
 
 /// Clear an OTP slot.
-pub fn otp_clear(
-  ctx: &mut args::ExecCtx<'_>,
-  slot: u8,
-  algorithm: args::OtpAlgorithm,
-) -> Result<()> {
+pub fn otp_clear(ctx: &mut ExecCtx<'_>, slot: u8, algorithm: args::OtpAlgorithm) -> Result<()> {
   with_device(ctx, |ctx, device| {
     let mut device = authenticate_admin(ctx, device)?;
     match algorithm {
@@ -747,7 +740,7 @@ pub fn otp_clear(
 }
 
 fn print_otp_status(
-  ctx: &mut args::ExecCtx<'_>,
+  ctx: &mut ExecCtx<'_>,
   algorithm: args::OtpAlgorithm,
   device: &nitrokey::DeviceWrapper<'_>,
   all: bool,
@@ -781,7 +774,7 @@ fn print_otp_status(
 }
 
 /// Print the status of the OTP slots.
-pub fn otp_status(ctx: &mut args::ExecCtx<'_>, all: bool) -> Result<()> {
+pub fn otp_status(ctx: &mut ExecCtx<'_>, all: bool) -> Result<()> {
   with_device(ctx, |ctx, device| {
     println!(ctx, "alg\tslot\tname")?;
     print_otp_status(ctx, args::OtpAlgorithm::Hotp, &device, all)?;
@@ -791,13 +784,10 @@ pub fn otp_status(ctx: &mut args::ExecCtx<'_>, all: bool) -> Result<()> {
 }
 
 /// Clear the PIN stored by various operations.
-pub fn pin_clear(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
+pub fn pin_clear(ctx: &mut ExecCtx<'_>) -> Result<()> {
   with_device(ctx, |_ctx, device| {
-    pinentry::clear(&pinentry::PinEntry::from(
-      pinentry::PinType::Admin,
-      &device,
-    )?)?;
-    pinentry::clear(&pinentry::PinEntry::from(pinentry::PinType::User, &device)?)?;
+    pinentry::clear(&pinentry::PinEntry::from(args::PinType::Admin, &device)?)?;
+    pinentry::clear(&pinentry::PinEntry::from(args::PinType::User, &device)?)?;
     Ok(())
   })
 }
@@ -806,20 +796,16 @@ pub fn pin_clear(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
 ///
 /// If the user has set the respective environment variable for the
 /// given PIN type, it will be used.
-fn choose_pin(
-  ctx: &mut args::ExecCtx<'_>,
-  pin_entry: &pinentry::PinEntry,
-  new: bool,
-) -> Result<String> {
+fn choose_pin(ctx: &mut ExecCtx<'_>, pin_entry: &pinentry::PinEntry, new: bool) -> Result<String> {
   let new_pin = match pin_entry.pin_type() {
-    pinentry::PinType::Admin => {
+    args::PinType::Admin => {
       if new {
         &ctx.new_admin_pin
       } else {
         &ctx.admin_pin
       }
     }
-    pinentry::PinType::User => {
+    args::PinType::User => {
       if new {
         &ctx.new_user_pin
       } else {
@@ -839,7 +825,7 @@ fn choose_pin(
 }
 
 /// Change a PIN.
-pub fn pin_set(ctx: &mut args::ExecCtx<'_>, pin_type: pinentry::PinType) -> Result<()> {
+pub fn pin_set(ctx: &mut ExecCtx<'_>, pin_type: args::PinType) -> Result<()> {
   with_device(ctx, |ctx, mut device| {
     let pin_entry = pinentry::PinEntry::from(pin_type, &device)?;
     let new_pin = choose_pin(ctx, &pin_entry, true)?;
@@ -849,8 +835,8 @@ pub fn pin_set(ctx: &mut args::ExecCtx<'_>, pin_type: pinentry::PinType) -> Resu
       &pin_entry,
       "Could not change the PIN",
       |current_pin| match pin_type {
-        pinentry::PinType::Admin => device.change_admin_pin(&current_pin, &new_pin),
-        pinentry::PinType::User => device.change_user_pin(&current_pin, &new_pin),
+        args::PinType::Admin => device.change_admin_pin(&current_pin, &new_pin),
+        args::PinType::User => device.change_user_pin(&current_pin, &new_pin),
       },
     )?;
 
@@ -862,11 +848,11 @@ pub fn pin_set(ctx: &mut args::ExecCtx<'_>, pin_type: pinentry::PinType) -> Resu
 }
 
 /// Unblock and reset the user PIN.
-pub fn pin_unblock(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
+pub fn pin_unblock(ctx: &mut ExecCtx<'_>) -> Result<()> {
   with_device(ctx, |ctx, mut device| {
-    let pin_entry = pinentry::PinEntry::from(pinentry::PinType::User, &device)?;
+    let pin_entry = pinentry::PinEntry::from(args::PinType::User, &device)?;
     let user_pin = choose_pin(ctx, &pin_entry, false)?;
-    let pin_entry = pinentry::PinEntry::from(pinentry::PinType::Admin, &device)?;
+    let pin_entry = pinentry::PinEntry::from(args::PinType::Admin, &device)?;
 
     try_with_pin(
       ctx,
@@ -878,7 +864,7 @@ pub fn pin_unblock(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
 }
 
 fn print_pws_data(
-  ctx: &mut args::ExecCtx<'_>,
+  ctx: &mut ExecCtx<'_>,
   description: &'static str,
   result: result::Result<String, nitrokey::Error>,
   quiet: bool,
@@ -911,7 +897,7 @@ fn check_slot(pws: &nitrokey::PasswordSafe<'_, '_>, slot: u8) -> Result<()> {
 
 /// Read a PWS slot.
 pub fn pws_get(
-  ctx: &mut args::ExecCtx<'_>,
+  ctx: &mut ExecCtx<'_>,
   slot: u8,
   show_name: bool,
   show_login: bool,
@@ -937,7 +923,7 @@ pub fn pws_get(
 
 /// Write a PWS slot.
 pub fn pws_set(
-  ctx: &mut args::ExecCtx<'_>,
+  ctx: &mut ExecCtx<'_>,
   slot: u8,
   name: &str,
   login: &str,
@@ -951,7 +937,7 @@ pub fn pws_set(
 }
 
 /// Clear a PWS slot.
-pub fn pws_clear(ctx: &mut args::ExecCtx<'_>, slot: u8) -> Result<()> {
+pub fn pws_clear(ctx: &mut ExecCtx<'_>, slot: u8) -> Result<()> {
   with_password_safe(ctx, |_ctx, mut pws| {
     pws
       .erase_slot(slot)
@@ -960,7 +946,7 @@ pub fn pws_clear(ctx: &mut args::ExecCtx<'_>, slot: u8) -> Result<()> {
 }
 
 fn print_pws_slot(
-  ctx: &mut args::ExecCtx<'_>,
+  ctx: &mut ExecCtx<'_>,
   pws: &nitrokey::PasswordSafe<'_, '_>,
   slot: usize,
   programmed: bool,
@@ -981,7 +967,7 @@ fn print_pws_slot(
 }
 
 /// Print the status of all PWS slots.
-pub fn pws_status(ctx: &mut args::ExecCtx<'_>, all: bool) -> Result<()> {
+pub fn pws_status(ctx: &mut ExecCtx<'_>, all: bool) -> Result<()> {
   with_password_safe(ctx, |ctx, pws| {
     let slots = pws
       .get_slot_status()
